/*
 * Project name:
     MotionSensor_GSM_ALARM
 * Copyright:
     (c) MikroElektronika, 2005-2011.
  * Revision History:
     20110413:
       - initial release (SZ);
 * Description:
     This is a simple project which demonstrates the use of SmartGM862 Board, Motion Sensor Board
     and our Start USB Board. ac:Hardware_used_in_example
     After a movement detection an alarming SMS message is being sent to number 123456789.
     Then, there is a 1 minute delay between detecting another movement and sending another SMS message.
 * Test configuration:
     MCU:             PIC18F2550
                      http://ww1.microchip.com/downloads/en/DeviceDoc/39632e.pdf
     Dev.Board:       Start USB Board (with USB HID Bootloader)
                      http://www.mikroe.com/eng/products/view/631/startusb-for-pic/
                      ac:StartUSB
     Oscillator:      HS-PLL, 48.00000 MHz
     Ext. Modules:    SmartGM862 Board
                      http://www.mikroe.com/eng/products/view/469/smartgm862-gps-board/
                      ac:SmartGM862
                      Motion Sensor Board
                      http://www.mikroe.com/eng/products/view/173/motion-sensor-board/
                      ac:MotionSensor
     SW:              mikroC PRO for PIC
                      http://www.mikroe.com/eng/products/view/7/mikroc-pro-for-pic/
 * NOTES:
     - Start USB Board: Place the J2 jumper (we'll use the RB1 LED to indicate that movement is detected).
     - SmartGM862-GPS extra board: enable only SW1.1 and SW1.5 switches.
       Also, connect PIC18F2550 (RC6, RC7, Vcc and Gnd) to SmartGM862 Board (PIN6, PIN7, Vcc and GND) respectively.
     - Connect the OUT pin of the Motion Sensor Board to the RB0 pin of PIC18F2550.
     - Use Wire Jumpers to establish connections:
       http://www.mikroe.com/eng/products/view/495/wire-jumpers/
*/

// Set of AT commands
const char atc0[] = "AT";                        // Every AT command starts with "AT"
const char atc1[] = "ATE0";                      // Disable command echo
const char atc2[] = "AT+CMGF=1";                 // TXT messages
const char atc3[] = "AT+CMGS=\"123456789\"";     // Send message to cell number : 123456789 (Enter your number instead of 123456789!)

// Alarm message
const char msg1[] = "Warning: Motion detected!!!";
//

// Responses to parse
const GSM_OK = 0;
const GSM_Ready_To_Receive_Message = 1;

sbit Motion_Sensor_Input at RB0_bit;
sbit Motion_Sensor_Input_Direction at TRISB0_bit;
sbit Movement_Detected_LED at RB1_bit;
sbit Movement_Detected_LED_Direction at TRISB1_bit;

char gsm_state = 0;
char response_rcvd = 0;
short responseID = -1, response = -1;

// uart rx interrupt handler
void interrupt(){
char tmp;
  if (PIR1.RCIF == 1) {                          // Do we have uart rx interrupt request?
    tmp = UART1_Read();                          // Get received byte

// process reception through state machine
// we are parsing only "OK" and "> " responses
    switch (gsm_state) {
      case  0: {
                response = -1;                   // Clear response
                if (tmp == 'O')                  // We have 'O', it could be "OK"
                  gsm_state = 1;                 // Expecting 'K'
                if (tmp == '>')                  // We have '>', it could be "> "
                  gsm_state = 10;                // Expecting ' '
                break;
               }
      case  1: {
                if (tmp == 'K') {                // We have 'K' ->
                  response = GSM_OK;             // We have "OK" response
                  gsm_state = 20;                // Expecting CR+LF
                }
                else
                  gsm_state = 0;                 // Reset state machine
                break;
               }
      case 10: {
                if (tmp == ' ') {
                  response_rcvd = 1;             // We have "> " response
                  response = GSM_Ready_To_Receive_Message; // Set reception flag
                  responseID = response;         // Set response ID
                }
                gsm_state = 0;                   // Reset state machine
                break;
                }

      case 20: {
                if (tmp == 13)                   // We have 13, it could be CR+LF
                  gsm_state = 21;                // Expecting LF
                else
                  gsm_state = 0;                 // Reset state machine
                break;
               }
      case 21: {
                if (tmp == 10) {                 // We have LF, response is complete
                  response_rcvd = 1;             // Set reception flag
                  responseID = response;         // Set response ID
                }
                gsm_state = 0;                   // Reset state machine
                break;
               }
      default: {                                 // Unwanted character
                gsm_state = 0;                   // Reset state machine
                break;
               }
    }
  }
}

// Send command or data to the Telit GM862 Module
void GM862_Send(const char *s)
{
// Send command or data string
   while(*s) {
      UART1_Write(*s++);
   }
// Terminatation by CR
   UART1_Write(0x0D);
}

// Get GSM response, if there is any
short Get_response() {
  if (response_rcvd) {
    response_rcvd = 0;
    return responseID;
  }
  else
    return -1;
}

// Wait for GSM response
void Wait_response(char rspns) {
  while (Get_response() != rspns)
    ;
}

// Send alarming SMS!
void Send_Alarm_Msg(){
       GM862_Send(atc2);
       Wait_response(GSM_OK);

       GM862_Send(atc3);
       Wait_response(GSM_Ready_To_Receive_Message);

       GM862_Send(msg1);
       UART1_Write(0x1A);
       UART1_Write(0x0D);
       Wait_response(GSM_OK);
}

// 1 min delay - minal time interval between sending SMSs
void Wait(){
char i;
    for(i = 0; i < 60; i++){
      Delay_ms(1000);
    }
}


void main() {

  WDTCON = 0;
  ADCON1 |= 0x0F;                                // Configure all ports with analog function as digital
  CMCON  |= 7;                                   // Disable comparators

// Enable Uart Rx interrupt
  PIE1.RCIE   = 1;
  INTCON.PEIE = 1;
  INTCON.GIE  = 1;

  Motion_Sensor_Input_Direction = 1;             // This pin will be used to detect the Motion Sensor state
  Movement_Detected_LED = 0;
  Movement_Detected_LED_Direction = 0;

  UART1_Init(19200);                             // Initialize USART module
  Delay_ms(100);

// Negotiate baud rate
  while(1) {
    GM862_Send(atc0);                            // Send "AT" string until GSM862 sets up its baud rade
    Delay_ms(100);                               // and gets it correctly
    if (Get_response() == GSM_OK)                // If GSM862 says "OK" on our baud rate we program can continue
      break;
  }

// Disable command echo
 GM862_Send(atc1);
 Wait_response(GSM_OK);

  Wait();                                        // Wait a while till there is no movement!

 while(1){
  if(Motion_Sensor_Input) {                      // If motion was detected than Alarm message is being sent!

    Movement_Detected_LED = 1;
    Send_Alarm_Msg();

    if(Motion_Sensor_Input);
    Movement_Detected_LED = 0;
    Wait();                                      // Next alarming SMS if the movement is present can be sent in 1 minute!
   }
 }
}